@@ -76,8 +76,9 @@ double text_correlation(mat pij, mat imat, mat jmat, double mean_haralick, doubl
 //' @param n_grey number of grey levels to use in texture calculation
 //' @param window_dims 2 element list with row and column dimensions of the
 //' texture window
-//' @param shift a length 2 vector with the number of cells to shift when
-//' computing co-ocurrency matrices
+//' @param shift a matrix where each row gives an (x, y) shift to use when 
+//' computing co-occurrency matrices. Textures will be calculated for each 
+//' shift, and the average over all shifts will be returned.
 //' @param statistics a list of strings naming the texture statistics to 
 //' calculate
 //' @param na_opt one of "ignore", "center", or "any"
@@ -87,17 +88,25 @@ double text_correlation(mat pij, mat imat, mat jmat, double mean_haralick, doubl
 //' parameter, containing the selected textures measures
 // [[Rcpp::export]]
 arma::cube calc_texture(arma::mat rast,
-        int n_grey, arma::vec window_dims, arma::vec shift,
+        int n_grey, arma::rowvec window_dims, arma::mat shift,
         Rcpp::CharacterVector statistics, std::string na_opt, double na_val) {
     mat imat(n_grey, n_grey);
     mat jmat(n_grey, n_grey);
     mat base_window(window_dims(0), window_dims(1));
     mat offset_window(window_dims(0), window_dims(1));
     mat pij(n_grey, n_grey);
-    vec base_ul(2), offset_ul(2), center_coord(2), lr_coord(2);
+    mat base_ul(shift.n_rows, 2, fill::zeros);
+    mat offset_ul(shift.n_rows, 2), center_coord(shift.n_rows, 2), 
+        lr_coord(shift.n_rows, 2);
     double mean_haralick, ENVI_mean;
 
-    // textures cube will hold the calculated texture statistics
+    // textures_temp mat will hold the texture statistics as calculated for 
+    // each shift, with textures in rows and per-shift values of each texture 
+    // in columns
+    mat textures_temp(statistics.size(), shift.n_rows);
+
+    // textures cube will hold the final calculated texture statistics 
+    // (averaged over all shifts as required)
     cube textures(rast.n_rows, rast.n_cols, statistics.size());
 
     std::map<std::string, double (*)(mat, mat, mat, double, double)> stat_func_map;
@@ -113,24 +122,34 @@ arma::cube calc_texture(arma::mat rast,
     stat_func_map["correlation"] = text_correlation;
 
     // Calculate the base upper left (ul) coords and offset upper left coords 
-    // as row, column with zero based indices.
-    base_ul = vec("0 0");
-    if (shift[0] < 0) {
-        base_ul[0] = base_ul[0] + abs(shift[0]);
+    // for each shift, as row, column with zero based indices.
+    for(unsigned n=0; n < shift.n_rows; n++) {
+        if (shift(n, 0) < 0) {
+            base_ul(n, 0) = base_ul(n, 0) + abs(shift(n, 0));
+        }
+        if (shift(n, 1) < 0) {
+            base_ul(n, 1) = base_ul(n, 1) + abs(shift(n, 1));
+        }
+        offset_ul.row(n) = base_ul.row(n) + shift.row(n);
+        center_coord.row(n) = base_ul.row(n) + floor(window_dims / 2);
+        // lr_coord gives the maximum lower right coordinate included in 
+        // the combined area of the offset and base windows
+        lr_coord.row(n) = window_dims + abs(shift.row(n)) - 1;
     }
-    if (shift[1] < 0) {
-        base_ul[1] = base_ul[1] + abs(shift[1]);
-    }
-    offset_ul = base_ul + shift;
-    center_coord = base_ul + floor(window_dims / 2);
-    // lr_coord gives the maximum lower right coordinate included in 
-    // the combined area of the offset and base windows
-    lr_coord = window_dims + abs(shift) - 1;
 
-    // Rcpp::Rcout << "base_ul: (" << base_ul(0) << ", " << base_ul(1) << ")" << std::endl;
-    // Rcpp::Rcout << "offset_ul: (" << offset_ul(0) << ", " << offset_ul(1) << ")" << std::endl;
-    // Rcpp::Rcout << "center_coord: (" << center_coord(0) << ", " << center_coord(1) << ")" << std::endl;
-    // Rcpp::Rcout << "lr_coord: (" << lr_coord(0) << ", " << lr_coord(1) << ")" << std::endl;
+    // Make base_ul and offset_ul relative to center_coord of 0, 0
+    base_ul = base_ul - center_coord;
+    offset_ul = offset_ul - center_coord;
+    lr_coord = lr_coord - center_coord;
+
+    // Rcpp::Rcout << "base_ul:" << std::endl;
+    // base_ul.print();
+    // Rcpp::Rcout << "offset_ul:" << std::endl;
+    // offset_ul.print();
+    // Rcpp::Rcout << "center_coord:" << std::endl;
+    // center_coord.print();
+    // Rcpp::Rcout << "lr_coord:" << std::endl;
+    // lr_coord.print();
      
     // Make a matrix of i's and a matrix of j's to be used in the below matrix 
     // calculations. These matrices are the same shape as pij with the entries 
@@ -145,83 +164,88 @@ arma::cube calc_texture(arma::mat rast,
     imat = repmat(linspace<vec>(1, pij.n_rows, pij.n_rows), 1, pij.n_cols);
     jmat = trans(imat);
 
-    for(unsigned row=0; row <= (rast.n_rows - abs(shift(0)) - window_dims(0)); row++) {
-        for(unsigned col=0; col <= (rast.n_cols - abs(shift(1)) - window_dims(1)); col++) {
-            // // Debugging code for tracking the upper left corner coordinate 
-            // // of base and offset matrices
-            // umat dbg_base(rast.n_rows, rast.n_cols, fill::zeros);
-            // for(unsigned i=0; i < dbg_base.n_elem; i++) {
-            //     dbg_base(i) = i;
-            // }
-            // dbg_base(row + base_ul(0), col + base_ul(1)) = 555;
-            // dbg_base(row + offset_ul(0), col + offset_ul(1)) = 999;
-            // dbg_base.print("ul coords (555 is base, 999 is offset):");
+    // Rcpp::Rcout << "cube_size: (" << textures.n_rows << ", " <<textures.n_cols << ", " <<textures.n_slices << ")" << std::endl;
+    // Rcpp::Rcout << "ctr_x_limit: " << (rast.n_rows - max(lr_coord.col(0))) << std::endl;
+    // Rcpp::Rcout << "ctr_y_limit: " << (rast.n_cols - max(lr_coord.col(1))) << std::endl;
+    for(unsigned ctr_x=max(center_coord.col(0)); ctr_x < (rast.n_rows - max(lr_coord.col(0))); ctr_x++) {
+        for(unsigned ctr_y=max(center_coord.col(1)); ctr_y < (rast.n_cols - max(lr_coord.col(1))); ctr_y++) {
+            for(unsigned shift_num=0; shift_num < shift.n_rows; shift_num++) {
+                // // Debugging code for tracking the upper left corner coordinate 
+                // // of base and offset matrices
+                // umat dbg_base(rast.n_rows, rast.n_cols, fill::zeros);
+                // for(unsigned i=0; i < dbg_base.n_elem; i++) {
+                //     dbg_base(i) = i;
+                // }
+                // dbg_base(ctr_x + base_ul(shift_num, 0), ctr_y + base_ul(shift_num, 1)) = 555;
+                // dbg_base(ctr_x + offset_ul(shift_num, 0), ctr_y + offset_ul(shift_num, 1)) = 999;
+                // dbg_base.print("ul coords (555 is base, 999 is offset):");
 
-            base_window = rast.submat(row + base_ul(0),
-                                      col + base_ul(1),
-                                      row + base_ul(0) + window_dims(0) - 1,
-                                      col + base_ul(1) + window_dims(1) - 1);
+                base_window = rast.submat(ctr_x + base_ul(shift_num, 0),
+                                          ctr_y + base_ul(shift_num, 1),
+                                          ctr_x + base_ul(shift_num, 0) + window_dims(0) - 1,
+                                          ctr_y + base_ul(shift_num, 1) + window_dims(1) - 1);
 
-            offset_window = rast.submat(row + offset_ul(0),
-                                        col + offset_ul(1),
-                                        row + offset_ul(0) + window_dims(0) - 1,
-                                        col + offset_ul(1) + window_dims(1) - 1);
-            pij.fill(0);
+                offset_window = rast.submat(ctr_x + offset_ul(shift_num, 0),
+                                            ctr_y + offset_ul(shift_num, 1),
+                                            ctr_x + offset_ul(shift_num, 0) + window_dims(0) - 1,
+                                            ctr_y + offset_ul(shift_num, 1) + window_dims(1) - 1);
+                pij.fill(0);
 
-            if (na_opt == "any") {
-                // Return NA for all textures within this window if there are 
-                // any NA values within the base_window or the offset_window
-                bool na_flag=false;
-                for(unsigned i=0; i < base_window.n_elem; i++) {
-                    if ((!arma::is_finite(base_window(i))) || (!arma::is_finite(offset_window(i)))) {
-                        for(signed s=0; s < statistics.size(); s++) {
-                            textures(row + center_coord(0),
-                                     col + center_coord(1), s) = na_val;
+                if (na_opt == "any") {
+                    // Return NA for all textures within this window if there are 
+                    // any NA values within the base_window or the offset_window
+                    bool na_flag=false;
+                    for(unsigned i=0; i < base_window.n_elem; i++) {
+                        if ((!arma::is_finite(base_window(i))) || (!arma::is_finite(offset_window(i)))) {
+                            for(signed s=0; s < statistics.size(); s++) {
+                                textures_temp(s, shift_num) = na_val;
+                            }
+                            na_flag = true;
+                            break;
                         }
-                        na_flag = true;
-                        break;
+                    }
+                    if (na_flag) continue;
+                } else if (na_opt == "center") {
+                    // Return NA for all textures within this window if the center 
+                    // value is an NA
+                    if (!arma::is_finite(base_window(center_coord(0), center_coord(1)))) {
+                        for(signed s=0; s < statistics.size(); s++) {
+                            textures_temp(s, shift_num) = na_val;
+                        }
+                        continue;
                     }
                 }
-                if (na_flag) continue;
-            } else if (na_opt == "center") {
-                // Return NA for all textures within this window if the center 
-                // value is an NA
-                if (!arma::is_finite(base_window(center_coord(0), center_coord(1)))) {
-                    for(signed s=0; s < statistics.size(); s++) {
-                        textures(row + center_coord(0),
-                                 col + center_coord(1), s) = na_val;
+
+                for(unsigned i=0; i < base_window.n_elem; i++) {
+                    if (!(arma::is_finite(base_window(i)) || arma::is_finite(offset_window(i)))) {
+                        // This will execute only if there is an NA in the window 
+                        // AND na_opt is set to "ignore" or "center"
+                        continue;
                     }
-                    continue;
+                    // Subtract one from the below indices to correct for row and 
+                    // col indices starting at 0 in C++ versus 1 in R.
+                    pij(base_window(i) - 1, offset_window(i) - 1)++;
+                }
+                pij = pij / base_window.n_elem;
+
+                mean_haralick = (mean(linspace<colvec>(1, pij.n_rows, pij.n_rows) %
+                            sum(pij, 1)) +
+                        mean(linspace<rowvec>(1, pij.n_rows, pij.n_rows) %
+                            sum(pij, 0))) / 2;
+                ENVI_mean = mean(vectorise(base_window) - 1);
+
+                // Loop over the selected statistics, using the stat_func_map map 
+                // to map each selected statistic to the appropriate texture 
+                // function.
+                for(signed i=0; i < statistics.size(); i++) {
+                    pfunc f = stat_func_map[Rcpp::as<std::string>(statistics(i))];
+                    textures_temp(i, shift_num) = (*f)(pij, imat, jmat, mean_haralick, ENVI_mean);
                 }
             }
-
-            for(unsigned i=0; i < base_window.n_elem; i++) {
-                if (!(arma::is_finite(base_window(i)) || arma::is_finite(offset_window(i)))) {
-                    // This will execute only if there is an NA in the window 
-                    // AND na_opt is set to "ignore" or "center"
-                    continue;
-                }
-                // Subtract one from the below indices to correct for row and 
-                // col indices starting at 0 in C++ versus 1 in R.
-                pij(base_window(i) - 1, offset_window(i) - 1)++;
-            }
-            pij = pij / base_window.n_elem;
-
-            mean_haralick = (mean(linspace<colvec>(1, pij.n_rows, pij.n_rows) %
-                        sum(pij, 1)) +
-                    mean(linspace<rowvec>(1, pij.n_rows, pij.n_rows) %
-                        sum(pij, 0))) / 2;
-            ENVI_mean = mean(vectorise(base_window) - 1);
-
-            // Loop over the selected statistics, using the stat_func_map map 
-            // to map each selected statistic to the appropriate texture 
-            // function.
+            // Average the statistics and across the shifts
             for(signed i=0; i < statistics.size(); i++) {
-                pfunc f = stat_func_map[Rcpp::as<std::string>(statistics(i))];
-                textures(row + center_coord(0),
-                         col + center_coord(1), i) = (*f)(pij, imat, jmat, mean_haralick, ENVI_mean);
+                textures(ctr_x, ctr_y, i) = mean(textures_temp.row(i));
             }
-
         }
     }
 
@@ -229,10 +253,11 @@ arma::cube calc_texture(arma::mat rast,
     // textures cannot be calculated due to edge effects. This saves 
     // computation time by avoiding the need to use fill::zeros when the matrix 
     // is initialized at the beginning of the function.
+    //
     
     // Fill nan values on left border
     for(unsigned row=0; row < rast.n_rows; row++) {
-        for(unsigned col=0; col < center_coord(1); col++) {
+        for(unsigned col=0; col < max(center_coord.col(1)); col++) {
             for(signed i=0; i < statistics.size(); i++) {
                 textures(row, col, i) = na_val;
             }
@@ -240,7 +265,7 @@ arma::cube calc_texture(arma::mat rast,
     }
 
     // Fill nan values on top border
-    for(unsigned row=0; row < center_coord(0); row++) {
+    for(unsigned row=0; row < max(center_coord.col(0)); row++) {
         for(unsigned col=0; col < rast.n_cols; col++) {
             for(signed i=0; i < statistics.size(); i++) {
                 textures(row, col, i) = na_val;
@@ -250,7 +275,7 @@ arma::cube calc_texture(arma::mat rast,
 
     // Fill nan values on right border
     for(unsigned row=0; row < rast.n_rows; row++) {
-        for(unsigned col=(rast.n_cols - (lr_coord(1) - center_coord(1))); col < rast.n_cols; col++) {
+        for(unsigned col=(rast.n_cols - max(lr_coord.col(1))); col < rast.n_cols; col++) {
             for(signed i=0; i < statistics.size(); i++) {
                 textures(row, col, i) = na_val;
             }
@@ -258,7 +283,7 @@ arma::cube calc_texture(arma::mat rast,
     }
 
     // Fill nan values on bottom border
-    for(unsigned row=(rast.n_rows - (lr_coord(0) - center_coord(0))); row < rast.n_rows; row++) {
+    for(unsigned row=(rast.n_rows - max(lr_coord.col(0))); row < rast.n_rows; row++) {
         for(unsigned col=0; col < rast.n_cols; col++) {
             for(signed i=0; i < statistics.size(); i++) {
                 textures(row, col, i) = na_val;
